/* visits.c - here are the functions that 'visit' servers and groups
   Copyright (C) 2000 Rik Snel <snel@phys.uu.nl>
   Parts copyright by Eric S. Raymond and the Free Software Foundation

   This file is part of newsleech.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 
*/
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<errno.h>

#include "newsleech.h"
#include "nntp.h"
#include "visits.h"

int visit_articles(FILE *fp, char *command, int sock, int *articles, int count, int aretry) {
	int c, error, totalerror;
	char cmd[100];
	for (c = 0; c < count; c++) {
		sprintf(cmd, "%s %d", command, articles[c]);
		while (aretry >= 0) {
			if ( (error = NNTP_ARTICLE_and_friends(sock, cmd, fp)) != -1) {
				break;
			}
			aretry -= 1;
		}
		totalerror += error;
	}
	return(totalerror);
}

int get_list(FILE *fp, char *header, int sock, int *articles, int count) {
	int error;
	char cmd[100];
	if (count == 1) sprintf(cmd, "XHDR %s %d", header, articles[0]);
	if (count == 2) {
		if (articles[1] == -1) sprintf(cmd, "XHDR %s %d-", header, articles[0]);
		else sprintf(cmd, "XHDR %s %d-%d", header, articles[0], articles[1]);
	}
	error += NNTP_ARTICLE_and_friends(sock, cmd, fp);
	return(error);
}

/* Please do not read this function unless you are a C compiler; 
   I wasn't feeling well when I wrote it.
*/
int visit_postings(int sock, struct postinginfo *postings, int count, int first, int last, int aretry) {
	int c, error;
	FILE *fp;
	for (c = 0; c < count; c++) {
		switch (postings[c].action) {
			case 's':
				fp = stdout;
				if (strcmp("XHDR", postings[c].command) == 0) {
					if (postings[c].articles[0] == -1) {
						postings[c].articles[0] = first;
						postings[c].articles[1] = last;
					}
					error += get_list(fp, postings[c].header, sock, postings[c].articles, postings[c].count);
				} else {
					error += visit_articles(fp, postings[c].command, sock, postings[c].articles, postings[c].count, aretry);
				}
				break;
			case 'f':
				if ((fp = fopen(postings[c].filename, "w")) != NULL) {
					if (verbose & 8) fprintf(stderr, "Opening %s for writing.\n", postings[c].filename);
					if (strcmp("XHDR", postings[c].command) == 0) {
						if (postings[c].articles[0] == -1) {
							postings[c].articles[0] = first;
							postings[c].articles[1] = last;
						}
						error += get_list(fp, postings[c].header, sock, postings[c].articles, postings[c].count);
					} else {
						error += visit_articles(fp, postings[c].command, sock, postings[c].articles, postings[c].count, aretry);
					}
					if (fclose(fp) != 0) fprintf(stderr, "Error %d closing %s.\n", errno, postings[c].filename);
					else if (verbose & 8) fprintf(stderr, "Closing %s.\n", postings[c].filename);
				} else {
					fprintf(stderr, "Error %d opening %s.\n", errno, postings[c].filename);
					error -= 1;
				}
				break;
			case 'p':
				if ((fp = popen(postings[c].pipeto, "w")) != NULL) {
					if (verbose & 8) fprintf(stderr, "Opening pipe to and executing %s.\n", postings[c].pipeto);	
					if (strcmp("XHDR", postings[c].command) == 0) {
						if (postings[c].articles[0] == -1) {
							postings[c].articles[0] = first;
							postings[c].articles[1] = last;
						}
						error += get_list(fp, postings[c].header, sock, postings[c].articles, postings[c].count);
					} else {
						error += visit_articles(fp, postings[c].command, sock, postings[c].articles, postings[c].count, aretry);
					}
					pclose(fp);
				} else {
					fprintf(stderr, "Error opening pipe to %s.\n", postings[c].pipeto);
					error -= 1;
				}	
				break;
			default:
				fprintf(stderr, "Action '%c' not implemented.\n", postings[c].action);
		}
	}
	return(error);
}		 

int visit_groups(int sock, struct groupinfo *groups, int count, int aretry) {
	int c, error;
	for (c = 0; c < count; c++) {
		if (NNTP_GROUP(sock, &groups[c]) < 0) {
			error -= 1;
		} else {
			error += visit_postings(sock, groups[c].postings, groups[c].count, groups[c].first, groups[c].last, aretry);
		}
	}
	return(error);
}

int visit_servers(struct serverinfo *servers, int count) {
	int c, sock, error;
	for (c = 0; c < count; c++) {
		sock = NNTP_init(&servers[c]);
		if (sock >= 0) {
			error += visit_groups(sock, servers[c].groups, servers[c].count, servers[c].aretry);
			error += NNTP_QUIT(sock);
		} else error -= 1;
	}
	return(error);
}
